//-----------------------------------------------------------------------------
// File:	RacorX.cpp
//
// Desc:	bump mapping with per-pixel specular lighting
//
//			This example uses the Common File routines.
//
//			Last modification: March 27th, 2002
//
// Copyright (c) 1995-2000 Microsoft Corporation. All rights reserved.
// Copyright (c) 2002 wolf@direct3d.net All rights reserved.
//-----------------------------------------------------------------------------

#define STRICT
#include <Windows.h>
#include <tchar.h>
#include <d3dx8.h>
#include "D3DApp.h"
#include "D3DFont.h"
#include "DXUtil.h"

#include "trackball.h"

//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------
struct ShaderVertex
{
    FLOAT x,y,z;
    FLOAT u,v;
    FLOAT nx,ny,nz;
    FLOAT bx,by,bz;
};

//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the 
//       generic functionality needed in all Direct3D samples. CMyD3DApplication 
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
class CMyD3DApplication : public CD3DApplication
{
public:
    CMyD3DApplication();
	~CMyD3DApplication();

    CD3DFont* m_pFont;
    CD3DFont* m_pFontSmall;

    D3DXMATRIX  m_matWorld;
    D3DXMATRIX  m_matView;
    D3DXMATRIX  m_matProj;
    D3DXMATRIX  m_matPosition;

    D3DXVECTOR4 m_LightDir;
    BOOL m_bLMouseDown;
    BOOL m_bRMouseDown;
    FLOAT m_xPos, m_yPos;
    FLOAT m_oPosX, m_oPosY;

    BYTE  m_bKey[256];
    FLOAT m_fTheta;
    FLOAT m_fTheta2;

    BOOL m_bBump; // toggle bump map
	BOOL m_bPixelShader; // toggle pixel shader version
	BOOL m_bPS14Available;

    trackball_t m_pTrackball;	// trackball to turn the model
    bool m_bZoomDrag;			// zooming
    float m_fZoom;

    LPDIRECT3DVERTEXBUFFER8 m_pVertices;
    LPDIRECT3DINDEXBUFFER8	m_pIndexBuffer;

    INT m_iNumTriangles,m_iNumVertices;
    LPDIRECT3DTEXTURE8		m_pNormalMap;
    LPDIRECT3DTEXTURE8		m_pColorTexture;

    // our shaders
    DWORD m_dwBumpShader;
    DWORD m_dwPixShaderDot3;
	DWORD m_dwPixShaderDot314;
	DWORD m_dwPixShader;

    TCHAR m_cColorMap[64];
    TCHAR m_cHeightMap[64];
   
    virtual HRESULT OneTimeSceneInit();
    virtual HRESULT InitDeviceObjects();
    virtual HRESULT RestoreDeviceObjects();
    virtual HRESULT InvalidateDeviceObjects();
    virtual HRESULT DeleteDeviceObjects();
    virtual HRESULT FrameMove();
    virtual HRESULT Render();
    virtual LRESULT MsgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
    virtual HRESULT ConfirmDevice(D3DCAPS8*,DWORD,D3DFORMAT);
    virtual HRESULT FinalCleanup();

private:
    HRESULT LoadXFile(TCHAR*);
	HRESULT CreateVSFromCompiledFile (IDirect3DDevice8* m_pd3dDevice, 
													 DWORD* dwDeclaration,
													 TCHAR* strVSPath, 
													 DWORD* m_dwVS);
	HRESULT CreatePSFromCompiledFile (IDirect3DDevice8* m_pd3dDevice, 
													 TCHAR* strPSPath, 
													 DWORD* dwPS);

};

//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    CMyD3DApplication d3dApp;

    if( FAILED( d3dApp.Create( hInst ) ) )
        return 0;

    return d3dApp.Run();
}


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device intialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::ConfirmDevice( D3DCAPS8* pCaps, DWORD dwBehavior,
                                          D3DFORMAT Format )
{
    if( (dwBehavior & D3DCREATE_HARDWARE_VERTEXPROCESSING ) ||
        (dwBehavior & D3DCREATE_MIXED_VERTEXPROCESSING ) )
    {
        if( pCaps->VertexShaderVersion < D3DVS_VERSION(1,1) )
            return E_FAIL;
    }
    if( pCaps->PixelShaderVersion < D3DPS_VERSION(1,4) )
		m_bPS14Available = FALSE;

    if( pCaps->PixelShaderVersion < D3DPS_VERSION(1,1) )
         return E_FAIL;

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
CMyD3DApplication::CMyD3DApplication()
{
    D3DXMATRIX matT;
    m_fElapsedTime			= 0.0f;
    m_dwCreationWidth		= 800;   // Width used to create window
    m_dwCreationHeight		= 600;
    m_bUseDepthBuffer		= TRUE;
    m_bShowCursorWhenFullscreen = TRUE;
    m_strWindowTitle		= _T("Per-Pixel Specular Lighting");
    m_pFont					= new CD3DFont( _T("Arial"), 12, D3DFONT_BOLD );
    m_pFontSmall			= new CD3DFont( _T("Arial"),  9, D3DFONT_BOLD );
	memset (m_bKey, 0, sizeof (m_bKey));
    m_bLMouseDown			= FALSE;
    m_bRMouseDown			= FALSE;
	m_bBump					= TRUE;  
	m_bPixelShader			= FALSE;
    m_bZoomDrag				= false;
    m_fZoom					= -50.0f;			// initial zoom
    m_fTheta				= 0;
    m_fTheta2				= 0;
    m_LightDir				= D3DXVECTOR4( 0.0f, 0.0f, 1.0f, 0.0f );
	m_pVertices				= NULL;
	m_pIndexBuffer			= NULL;
	m_iNumTriangles			= 0;
	m_iNumVertices			= 0;
	m_pNormalMap			= NULL;
	m_pColorTexture			= NULL;
    D3DXMatrixIdentity(&m_matWorld);
    D3DXMatrixIdentity(&m_matPosition);
}

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application denstructor
//-----------------------------------------------------------------------------
CMyD3DApplication::~CMyD3DApplication()
{
	SAFE_DELETE(m_pFont);
	SAFE_DELETE(m_pFontSmall);
}


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: track mouse movement
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FinalCleanup()
{
    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Message proc function to handle key and menu input
//-----------------------------------------------------------------------------
LRESULT CMyD3DApplication::MsgProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    // Trap the context menu
    if( WM_CONTEXTMENU==uMsg )
        return 0;

    switch(uMsg)
    {
    case WM_LBUTTONDOWN:
		SetCapture(hwnd);
        return 0;
    case WM_LBUTTONUP:
		ReleaseCapture();
        return 0;

        //track right mouse button 
    case WM_RBUTTONDOWN:
        m_xPos = LOWORD(lParam);
        m_yPos = HIWORD(lParam);
    
        m_bRMouseDown = TRUE;
        break;

    case WM_RBUTTONUP:
        m_bRMouseDown = FALSE;
        break;

		//track mouse movement for point light
    case WM_MOUSEMOVE:
	{
        if(m_bRMouseDown)
        {
            m_xPos = LOWORD(lParam);
            m_yPos = HIWORD(lParam);
        }

	    RECT r;
		FLOAT dx,dy;
		D3DXMATRIX mat1,mat2,mat3;

		GetWindowRect(m_hWnd,&r);
	    dx = ((FLOAT)m_xPos)/((FLOAT)r.right-r.left) * 2 - 1.0f;
	    dy = ((FLOAT)m_yPos)/((FLOAT)r.right-r.left) * 2 - 0.7f;
    
	    // right mouse button moves the light around
	    if(m_bRMouseDown)
		{
	       m_LightDir.x = -dx;
	       m_LightDir.y = dy;
		   m_LightDir.z = 1.0f;
		}
    
	    // left button rotates the object
	    if(m_bLMouseDown)
		{   
	      m_fTheta  = -dy+m_oPosY;
	      m_fTheta2 = -dx+m_oPosX;
    
	      D3DXMatrixRotationX(&mat1,m_fTheta * 0.1f);
	      D3DXMatrixRotationY(&mat2,m_fTheta2 * 0.1f);
	      D3DXMatrixMultiply(&mat3,&mat1,&mat2);
	      D3DXMatrixMultiply(&m_matWorld,&m_matWorld,&mat3);
		}
	}
    break;

    case WM_KEYUP:
        //mark a key up
        m_bKey[wParam] = FALSE;
        break;

    case WM_KEYDOWN:
        //mark a key down
         m_bKey[wParam] = TRUE;
       break;
   }

    return CD3DApplication::MsgProc( hwnd, uMsg, wParam, lParam );
}



//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Animates the scene
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FrameMove()
{

	m_pTrackball.TrackMouse(m_hWnd, m_bZoomDrag, &m_fZoom); // track mouse movements and move object
    m_pTrackball.SetMatrix(m_matWorld); // get the world matrix for the trackball

    // setup view matrix
    D3DXVECTOR3 vUpVec(0,1,0);
    D3DXVECTOR3 vEyePt(0,0,m_fZoom);
    D3DXVECTOR3 vLookatPt(0,0,0);
    D3DXMatrixLookAtLH(&m_matView, &vEyePt, &vLookatPt, &vUpVec);

    D3DXVECTOR3 vecT = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
    if(m_bKey['W']) m_fZoom += 0.5f; // Move Forward
    if(m_bKey['S']) m_fZoom -= 0.5f; // Move Backward

	// toggle diffuse lighting
    if(m_bKey['B']) {m_bKey['B']=0; m_bBump = !m_bBump;}; 

	// toggle pixel shader
    if(m_bKey['P'] && m_bPS14Available) {m_bKey['P']=0; m_bPixelShader = !m_bPixelShader;}; 

	// light direction
    D3DXVec4Normalize(&m_LightDir,&m_LightDir);
    m_pd3dDevice->SetVertexShaderConstant(12, &m_LightDir,  1 );	// light direction

	// world * view * proj matrix
    D3DXMATRIX  matTemp;
    D3DXMatrixTranspose(&matTemp,&(m_matWorld * m_matView * m_matProj));
    m_pd3dDevice->SetVertexShaderConstant(8, &matTemp, 4);	

	// world matrix
    D3DXMatrixTranspose(&matTemp,&m_matWorld);
    m_pd3dDevice->SetVertexShaderConstant(0, &matTemp, 4);	

	// eye vector
    m_pd3dDevice->SetVertexShaderConstant(24, &vEyePt, 1);		

   return S_OK;
}

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Renders the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::Render()
{
    m_pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 
						D3DCOLOR_ARGB( 0x00, 0x00, 0x00, 0x00), 1.0, 0);
   
    // Begin the scene
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
	    //  specular lighting. 
		if(m_bBump)
		{	
			m_pd3dDevice->SetTexture(0,m_pColorTexture);
			m_pd3dDevice->SetTexture(1,m_pNormalMap);    
			
			if (m_bPixelShader)
		        m_pd3dDevice->SetPixelShader(m_dwPixShaderDot314);
			else
		        m_pd3dDevice->SetPixelShader(m_dwPixShaderDot3);

		}
		else
		{
			//no lighting, just base color texture
	        m_pd3dDevice->SetTexture(0,m_pColorTexture);
	        m_pd3dDevice->SetPixelShader(m_dwPixShader);
		}

	    m_pd3dDevice->SetStreamSource( 0, m_pVertices, sizeof(ShaderVertex) );
		m_pd3dDevice->SetIndices(m_pIndexBuffer,0);
		m_pd3dDevice->SetVertexShader(m_dwBumpShader);
		m_pd3dDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST,0,m_iNumVertices,0,m_iNumTriangles);

		// Output statistics
		m_pFont->DrawText( 2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats );
		m_pFont->DrawText( 2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats );
		m_pFontSmall->DrawText( 2, 40, D3DCOLOR_ARGB(255,255,255,255), 
		TEXT("B -     Toggle Specular Lighting\nW -     Zoom In\nS -     Zoom Out") );
		if (m_bPixelShader && m_bPS14Available)
			m_pFontSmall->DrawText( 2, 85, D3DCOLOR_ARGB(255,255,255,255), 
									TEXT("P - Pixel Shader ps.1.4") );
		else if(m_bPS14Available)
			m_pFontSmall->DrawText( 2, 85, D3DCOLOR_ARGB(255,255,255,255), 
									TEXT("P - Pixel Shader ps.1.1") );
		else
			m_pFontSmall->DrawText( 2, 85, D3DCOLOR_ARGB(255,255,255,255), 
									TEXT("ps.1.4 Not Supported") );

        // End the scene.
        m_pd3dDevice->EndScene();
	}
    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::OneTimeSceneInit()

{
    // Set cursor to indicate that user can move the object with the mouse
#ifdef _WIN64
    SetClassLongPtr( m_hWnd, GCLP_HCURSOR, (LONG_PTR)LoadCursor( NULL, IDC_SIZEALL ) );
#else
    SetClassLong( m_hWnd, GCL_HCURSOR, (LONG)LoadCursor( NULL, IDC_SIZEALL ) );
#endif

    strcpy(m_cHeightMap, _T("media/earthbump.bmp"));
    strcpy(m_cColorMap,_T("media/earth.bmp"));

   return S_OK;
}


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InitDeviceObjects()
{
    D3DXMATRIX matT;

    m_pFont->InitDeviceObjects( m_pd3dDevice );
    m_pFontSmall->InitDeviceObjects( m_pd3dDevice );

    TCHAR strFile[160];
    DXUtil_FindMediaFile(strFile, _T("sphere.x"));
    LoadXFile(strFile);

	//
	// create bump map or normal map from height map
	//
    if(FAILED(D3DXCreateTextureFromFile(m_pd3dDevice, m_cColorMap,&m_pColorTexture)))
        return E_FAIL;

    LPDIRECT3DTEXTURE8 pHeightMap = NULL;
    if(FAILED(D3DXCreateTextureFromFile(m_pd3dDevice,m_cHeightMap,&pHeightMap)))
        return E_FAIL;

    D3DSURFACE_DESC    desc;
    pHeightMap->GetLevelDesc(0,&desc);
    if(FAILED(D3DXCreateTexture(m_pd3dDevice, desc.Width, desc.Height, 0, 0, 
								D3DFMT_A8R8G8B8,D3DPOOL_MANAGED, &m_pNormalMap)))
        return E_FAIL;

    D3DXComputeNormalMap(m_pNormalMap,pHeightMap, NULL, 0, D3DX_CHANNEL_RED,10);
	SAFE_RELEASE(pHeightMap);

	// vertex shader declaration
    DWORD dwDecl[] = 
    {
        D3DVSD_STREAM(0),
        D3DVSD_REG(0,   D3DVSDT_FLOAT3), // position
        D3DVSD_REG(3,   D3DVSDT_FLOAT3), // normal
        D3DVSD_REG(7,   D3DVSDT_FLOAT2), // texture
        D3DVSD_REG(8,   D3DVSDT_FLOAT3), // tangent
        D3DVSD_END()
    };

	CreateVSFromCompiledFile (m_pd3dDevice, dwDecl, "shaders/SpecDot3Pix.vso", &m_dwBumpShader);
	CreatePSFromCompiledFile (m_pd3dDevice, "shaders/SpecDot3.pso", &m_dwPixShaderDot3);
	CreatePSFromCompiledFile (m_pd3dDevice, "shaders/diff.pso", &m_dwPixShader);
	CreatePSFromCompiledFile (m_pd3dDevice, "SpecDot314.pso", &m_dwPixShaderDot314);

   return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RestoreDeviceObjects()
{
    m_pFont->RestoreDeviceObjects();
    m_pFontSmall->RestoreDeviceObjects();

    m_pd3dDevice->SetTextureStageState( 1, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    m_pd3dDevice->SetTextureStageState( 1, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);

    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE);    

    m_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

    m_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
    
    D3DXMATRIX matProjection;
    D3DXMatrixPerspectiveFovLH(&m_matProj, 0.05f, 
								(FLOAT)m_d3dsdBackBuffer.Width /(FLOAT)m_d3dsdBackBuffer.Height, 
								1.0f, 10000.0f);

	// init trackball 
	m_pTrackball.Init(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);

    // constant values
	D3DXVECTOR4 half(0.5f,0.5f,0.5f,0.5f);
    m_pd3dDevice->SetVertexShaderConstant(33, &half, 1);

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc:
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InvalidateDeviceObjects()
{
    m_pFont->InvalidateDeviceObjects();
    m_pFontSmall->InvalidateDeviceObjects();
    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::DeleteDeviceObjects()
{
    m_pFont->DeleteDeviceObjects();
    m_pFontSmall->DeleteDeviceObjects();

    SAFE_RELEASE(m_pIndexBuffer);
    SAFE_RELEASE(m_pVertices);

    SAFE_RELEASE(m_pNormalMap);
    SAFE_RELEASE(m_pColorTexture);
	
    if(m_dwBumpShader != 0xffffffff)
            m_pd3dDevice->DeleteVertexShader(m_dwBumpShader);
	if(m_dwPixShaderDot3 != 0xffffffff)
            m_pd3dDevice->DeletePixelShader(m_dwPixShaderDot3);
	if(m_dwPixShaderDot314 != 0xffffffff)
            m_pd3dDevice->DeletePixelShader(m_dwPixShaderDot314);
	if(m_dwPixShader != 0xffffffff)
            m_pd3dDevice->DeletePixelShader(m_dwPixShader);

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: LoadXFile()
// Desc: loads the x-file
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::LoadXFile(TCHAR* name)
{
    HRESULT		hr;
    LPD3DXMESH	pMeshSysMem = NULL,pMeshSysMem2 = NULL;
  
    if (FAILED (D3DXLoadMeshFromX(name, D3DXMESH_SYSTEMMEM, 
                          m_pd3dDevice, NULL, NULL, NULL, &pMeshSysMem)))
		return E_FAIL;

    // reform the mesh to one which will have space for the tangent vectors
	// D3DFVF_TEXCOORDSIZE3(1) indicates that a 3D texture is used in the
	// second texture stage
    hr = pMeshSysMem->CloneMeshFVF(D3DXMESH_MANAGED,  D3DFVF_XYZ | D3DFVF_NORMAL
                                    | D3DFVF_TEX2 | D3DFVF_TEXCOORDSIZE3(1), 
                                    m_pd3dDevice, &pMeshSysMem2);

    // compute the normals
    hr = D3DXComputeNormals(pMeshSysMem,NULL);
    if(FAILED(hr))
        return E_FAIL;

	// compute U (tangent)
	hr = D3DXComputeTangent(pMeshSysMem,0,pMeshSysMem2,1,D3DX_COMP_TANGENT_NONE,TRUE,NULL);
    if(FAILED(hr))
        return E_FAIL;
 
    hr = pMeshSysMem2->GetIndexBuffer(&m_pIndexBuffer);
    if(FAILED(hr))
        return E_FAIL;
    hr = pMeshSysMem2->GetVertexBuffer(&m_pVertices);
    if(FAILED(hr))
        return E_FAIL;

    m_iNumTriangles = pMeshSysMem2->GetNumFaces();
    m_iNumVertices = pMeshSysMem2->GetNumVertices();
    
    // cleanup
    SAFE_RELEASE(pMeshSysMem)
    SAFE_RELEASE(pMeshSysMem2)

  return S_OK;
}


//------------------------------------------------------------------------------
// Name:	CreateVSFromBinFile
// Desc:	loads a binary *.vso file 
//			and creates a vertex shader
//------------------------------------------------------------------------------
HRESULT CMyD3DApplication::CreateVSFromCompiledFile (IDirect3DDevice8* m_pd3dDevice, 
													 DWORD* dwDeclaration,
													 TCHAR* strVSPath, 
													 DWORD* m_dwVS)
{
	char szBuffer[128];		// debug output
	DWORD*	dwpVS;			// pointer to address space of the calling process
	HANDLE hFile, hMap;		// handle file and handle mapped file
    TCHAR tchTempVSPath[512];	// temporary file path
	HRESULT hr;				// error 

    if( FAILED( hr = DXUtil_FindMediaFile( tchTempVSPath, strVSPath ) ) )
        return D3DAPPERR_MEDIANOTFOUND;
	
	hFile = CreateFile(tchTempVSPath, GENERIC_READ,0,0,OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,0);

	if(hFile != INVALID_HANDLE_VALUE) 
	{
		if(GetFileSize(hFile,0) > 0) 
			hMap = CreateFileMapping(hFile,0,PAGE_READONLY,0,0,0);
		else
		{
			CloseHandle(hFile);
			return E_FAIL;		
		}
	}	
	else
		return E_FAIL;	
	
	// maps a view of a file into the address space of the calling process
	dwpVS = (DWORD *)MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0);
		
	// Create the vertex shader
	hr = m_pd3dDevice->CreateVertexShader( dwDeclaration, dwpVS, m_dwVS, 0 );
	if ( FAILED(hr) )
	{
		OutputDebugString( "Failed to create Vertex Shader, errors:\n" );
		D3DXGetErrorStringA(hr,szBuffer,sizeof(szBuffer));
		OutputDebugString( szBuffer );
		OutputDebugString( "\n" );
	  return hr;
	}
	
	UnmapViewOfFile(dwpVS);
	CloseHandle(hMap);
	CloseHandle(hFile);
	
  return S_OK;
}

//------------------------------------------------------------------------------
// Name:	CreatePSFromBinFile
// Desc:	loads a binary *.pso file 
//			and creates a pixel shader
//------------------------------------------------------------------------------
HRESULT CMyD3DApplication::CreatePSFromCompiledFile (IDirect3DDevice8* m_pd3dDevice, 
													 TCHAR* strPSPath, 
													 DWORD* dwPS)
{
	char szBuffer[128];		// debug output
	DWORD*	pdwPS;			// pointer to address space of the calling process
	HANDLE hFile, hMap;		// handle file and handle mapped file
    TCHAR tchTempVSPath[512];	// temporary file path
	HRESULT hr;				// error 

    if( FAILED( hr = DXUtil_FindMediaFile( tchTempVSPath, strPSPath ) ) )
        return D3DAPPERR_MEDIANOTFOUND;
	
	hFile = CreateFile(tchTempVSPath, GENERIC_READ,0,0,OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,0);

	if(hFile != INVALID_HANDLE_VALUE) 
	{
		if(GetFileSize(hFile,0) > 0) 
			hMap = CreateFileMapping(hFile,0,PAGE_READONLY,0,0,0);
		else
		{
			CloseHandle(hFile);
			return E_FAIL;		
		}
	}	
	else
		return E_FAIL;	
	
	// maps a view of a file into the address space of the calling process
	pdwPS = (DWORD *)MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0);
		
	// Create the pixel shader
	hr = m_pd3dDevice->CreatePixelShader(pdwPS, dwPS);
	if ( FAILED(hr) )
	{
		OutputDebugString( "Failed to create Pixel Shader, errors:\n" );
		D3DXGetErrorStringA(hr,szBuffer,sizeof(szBuffer));
		OutputDebugString( szBuffer );
		OutputDebugString( "\n" );
	  return hr;
	}
	
	UnmapViewOfFile(pdwPS);
	CloseHandle(hMap);
	CloseHandle(hFile);
	
  return S_OK;
}
